@@ -34,7 +34,7 @@ pub enum ConductorApiError {
 
     /// Conductor threw an error during API call.
     #[error("Conductor returned an error while using a ConductorApi: {0:?}")]
-    ConductorError(#[from] ConductorError),
+    ConductorError(#[from] Box<ConductorError>),
 
     /// Io error.
     #[error("Io error while using a Interface Api: {0:?}")]
@@ -119,6 +119,12 @@ impl ConductorApiError {
     }
 }
 
+impl From<ConductorError> for ConductorApiError {
+    fn from(conductor_api_error: ConductorError) -> Self {
+        Self::from(Box::new(conductor_api_error))
+    }
+}
+
 /// All the serialization errors that can occur
 #[derive(Error, Debug)]
 pub enum SerializationError {
