@@ -11,11 +11,11 @@ pub fn x_25519_x_salsa20_poly1305_decrypt(
     call_context: Arc<CallContext>,
     input: X25519XSalsa20Poly1305Decrypt,
 ) -> Result<Option<XSalsa20Poly1305Data>, WasmError> {
-    match HostFnAccess::from(&call_context.host_access()) {
-        HostFnAccess{ keystore: Permission::Allow, .. } => {
+    match HostFnAccess::from(&call_context.host_context()) {
+        HostFnAccess{ keystore_deterministic: Permission::Allow, .. } => {
             tokio_helper::block_forever_on(async move {
                 call_context
-                    .host_access
+                    .host_context
                     .keystore()
                     .x_25519_x_salsa20_poly1305_decrypt(input)
                     .await
